@@ -433,8 +433,13 @@ impl Readable for Created2 {
 /// in the [crate::relaycell] module.
 #[derive(Clone)]
 pub struct Relay {
-    // XXXX either this shouldn't be boxed, or RelayCellBody should be boxed!
     /// The contents of the relay cell as encoded for transfer.
+    ///
+    /// TODO(nickm): It's nice that this is boxed, since we don't want to copy
+    /// cell data all over the place. But unfortunately, a there are some other
+    /// places where we _don't_ Box things that we should, and more copies than
+    /// necessary happen. We should refactor our data handling until we're mostly
+    /// moving around pointers rather than copying data;  see ticket #7.
     body: Box<RawCellBody>,
 }
 impl Relay {
@@ -503,6 +508,10 @@ impl Destroy {
     pub fn new(reason: DestroyReason) -> Self {
         Destroy { reason }
     }
+    /// Return the provided reason for destroying the circuit.
+    pub fn reason(&self) -> DestroyReason {
+        self.reason
+    }
 }
 impl Body for Destroy {
     fn into_message(self) -> ChanMsg {
@@ -548,11 +557,33 @@ caret_int! {
         TIMEOUT = 10,
         /// Circuit was destroyed w/o client truncate (?)
         DESTROYED = 11,
-        /// Request for unknown hidden service
+        /// Request for unknown onion service
         NOSUCHSERVICE = 12
     }
 }
 
+impl DestroyReason {
+    /// Return a human-readable string for this reason.
+    pub fn human_str(&self) -> &'static str {
+        match *self {
+            DestroyReason::NONE => "No reason",
+            DestroyReason::PROTOCOL => "Protocol violation",
+            DestroyReason::INTERNAL => "Internal error",
+            DestroyReason::REQUESTED => "Client sent a TRUNCATE command",
+            DestroyReason::HIBERNATING => "Relay is hibernating and not accepting requests",
+            DestroyReason::RESOURCELIMIT => "Relay ran out of resources",
+            DestroyReason::CONNECTFAILED => "Couldn't connect to relay",
+            DestroyReason::OR_IDENTITY => "Connected to relay with different OR identity",
+            DestroyReason::CHANNEL_CLOSED => "The OR channels carrying this circuit died",
+            DestroyReason::FINISHED => "Circuit expired for being too dirty or old",
+            DestroyReason::TIMEOUT => "Circuit construction took too long",
+            DestroyReason::DESTROYED => "Circuit was destroyed without client truncate",
+            DestroyReason::NOSUCHSERVICE => "No such onion service",
+            _ => "Unrecognized reason",
+        }
+    }
+}
+
 /// The netinfo message ends channel negotiation.
 ///
 /// It tells the other party on the channel our view of the current time,
@@ -596,7 +627,7 @@ fn take_one_netinfo_addr(r: &mut Reader<'_>) -> Result<Option<IpAddr>> {
             Ok(Some(IpAddr::V4(bytes.into())))
         }
         (0x06, 16) => {
-            // XXXX is there a better way?
+            // TODO(nickm) is there a better way?
             let mut bytes = [0_u8; 16];
             (&mut bytes[..]).copy_from_slice(abody);
             Ok(Some(IpAddr::V6(bytes.into())))
@@ -755,10 +786,10 @@ pub struct PaddingNegotiate {
     /// Whether to start or stop padding
     command: u8,
     /// Suggested lower-bound value for inter-packet timeout in msec.
-    // XXXX is that right?
+    // TODO(nickm) is that right?
     ito_low_ms: u16,
     /// Suggested upper-bound value for inter-packet timeout in msec.
-    // XXXX is that right?
+    // TODO(nickm) is that right?
     ito_high_ms: u16,
 }
 impl PaddingNegotiate {
@@ -1141,3 +1172,17 @@ msg_into_cell!(Certs);
 msg_into_cell!(AuthChallenge);
 msg_into_cell!(Authenticate);
 msg_into_cell!(Authorize);
+
+#[cfg(test)]
+mod test {
+    use super::*;
+    #[test]
+    fn destroy_reason() {
+        let r1 = DestroyReason::CONNECTFAILED;
+
+        assert_eq!(r1.human_str(), "Couldn't connect to relay");
+
+        let r2 = DestroyReason::from(200); // not a specified number.
+        assert_eq!(r2.human_str(), "Unrecognized reason");
+    }
+}
