@@ -618,8 +618,7 @@ impl Body for Sendme {
                     Some(r.take(dlen as usize)?.into())
                 }
                 _ => {
-                    // XXXX is this an error?
-                    None
+                    return Err(Error::BadMessage("Unrecognized SENDME version."));
                 }
             }
         };
@@ -855,6 +854,10 @@ impl Truncated {
     pub fn new(reason: DestroyReason) -> Self {
         Truncated { reason }
     }
+    /// Get the provided reason to truncate the circuit.
+    pub fn reason(self) -> DestroyReason {
+        self.reason
+    }
 }
 impl Body for Truncated {
     fn into_message(self) -> RelayMsg {
